import unittest
from translit_me.transliterator import transliterate as tr
from translit_me.lang_tables import *


class TestTransliterate(unittest.TestCase):
    def test_hebrew_arabic(self):
        names = ['נועַם', "מאנץ'", "בישינה", "דימונה"]
        expected = ['نوعَم', 'مانض', 'بيشينة', 'بيسينة', 'ديمونة', 'ضيمونة']
        res = tr(names, HE_AR)
        print(res)
        self.assertListEqual(res, expected)

    def test_hebrew_english(self):
        names = ['נועַם', "מאנץ'", "בישינה"]  # ["מסטראי'",'משתראן']
        # ['כסבין', 'קלש', 'ארמילו', 'אבירו', 'בישינה', 'קדמות', 'לודקיא', 'גרדיגי', 'מיטילין', 'יובשטריסה']

        expected = ['nuʿam', 'noʿam', 'manḍ', 'manch', 'mānḍ', 'mānch', 'menḍ', 'mench', 'bīṣīna', 'bīṣīne', 'bīshīna',
                    'bīshīne', 'vīṣīna', 'vīṣīne', 'vīshīna', 'vīshīne']
        res = tr(names, HE_EN)
        print(res)
        self.assertListEqual(res, expected)

    def test_multi_word(self):
        names = ['כפר סבא', 'כפר סבא רבא', 'בן פורד יוסף']
        expected = ['كفر سبا', 'كفر سبا ربا', 'بن فورد يوسف', 'بن فورض يوسف']
        res = tr(names, HE_AR)
        print(res)
        self.assertListEqual(res, expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from translit_me.transliterator import remove_vowels as rv
from translit_me.lang_tables import *


class TestVowelRemover(unittest.TestCase):
    def test_hebrew(self):
        names = ['נועַם שגיא', "מאנץ'", "בישינה", "דימונה"]
        expected = ['נעם שג', "מנץ'", "בשנ", "דמנ"]
        res = []
        for name in names:
            res.append(rv(name, heb_vowels))
        print(res)
        self.assertListEqual(res, expected)

    def test_english(self):
        names = ['noam sagi', "manch'", "isengard"]
        expected = ['nm sg', "mnch'", 'isngrd']
        res = []
        for name in names:
            res.append(rv(name, en_vowels))
        print(res)
        self.assertListEqual(res, expected)

    def test_arabic(self):
        names = ['دَنْضِيل', "هُرْمُز اَرْداشير"]
        expected = ['دَنْضِيل', 'هُرْمُز اَرْداشير']
        res = []
        for name in names:
            res.append(rv(name, en_vowels))
        print(res)
        self.assertListEqual(res, expected)


if __name__ == '__main__':
    unittest.main()



def heh_ar(word, position):
    """
    Exception function for the letter heh in Hebrew.
    If at the end of the word, then use Ta Marbuta instead of Ta
    :param word: for testing if the letter is at the end
    :param position: for testing if the letter is at the end
    :return: Ta or Ta Marbuta
    """
    return 'ة' if position == len(word) - 1 else 'ه'


def heh_en(word, position):
    """
    Exception function for the letter heh in Hebrew.
    If at the end of the word, then use 'a' instead of 'h'
    :param word: for testing if the letter is at the end
    :param position: for testing if the letter is at the end
    :return: h or [a, e]
    """
    return ['a', 'e'] if position == len(word) - 1 else 'h'


# Set of final letter exception functions for Arabic --> Hebrew
def fin_kaf_ar(word, position):
    return ['ּך', 'ק'] if position == len(word) - 1 else ['כּ', 'ק']


def fin_mem_ar(word, position):
    return 'ם' if position == len(word) - 1 else 'מ'


def fin_nun_ar(word, position):
    return 'ן' if position == len(word) - 1 else 'נ'


def fin_zad_ar(word, position):
    return 'ץ' if position == len(word) - 1 else 'צ'


def fin_zhad_ar(word, position):
    return "ץ'" if position == len(word) - 1 else "צ'"


def fin_peh_ar(word, position):
    return 'ף' if position == len(word) - 1 else 'פ'


HE_AR = {"א": "ا", "ב": "ب", "בּ": "ب", "ג": "چ", "ג'": "ج", "ד": ["د", "ض"], "ד'": "ذ", "ה": heh_ar, "ו": "و",
         "ז": "ز",
         "ח": "ح", "ח'": "خ", "ט": ["ط", "ظ"], "י": "ي", "כּ": "ك", "ךּ": "ك", "כ": "ك", "ך": "ك", "ל": "ل", "מ": "م",
         "ם": "م", "נ": "ن",
         "ן": "ن",
         "ס": "س", "ע": "ع", "ע'": "غ", "פ": "ف", "ף": "ف", "פּ": "ب", "ףּ": "ب", "צ": "ص", "ץ": "ص", "צ'": "ض",
         "ץ'": "ض", "ק": "ق", "ר": "ر", "ש": ["ش", "س"], "ת": "ت", "ת'": "ث", "ָ": "َ", "ַ": "َ", "ִ": "ِ", "ֻ": "ُ",
         "וּ": "ُ", "-": "-", "'": "", "]": "", "׳": "", "[": "",
         "ְ": "ْ"}
# Done: duplicate key  "ד":"ض",
# Done: duplicate key "ט":"ظ",
# Done: Differ between dagesh kal and dagesh hazak
# Done: handle Shin/Sin markings if exist
# Done: handle Kamatz vowel


AR_HE = {"ال": "אל", "ا": "א", "إ": "א", "أ": "א", "آ": "א", "ب": "בּ", "چ": "ג", "ج": "ג'","א":"ؤ","א":"ئ", "ح": "ח", "خ": "ח'",
         "د": "ד", "ذ": "ד'", "ه": "ה", "ة": "ה", "ز": "ז", "ر": "ר", "ت": "ת", "ث": "ת'", "ط": "ט", "ظ": "ט'",
         "س": "ס", "ش": "ש", "ع": "ע", "غ": "ע'", "ص": fin_zad_ar, "ض": fin_zhad_ar, "ف": fin_peh_ar, "پ": "פּ",
         "ق": "ק", "ك": fin_kaf_ar, "ل": "ל", "م": fin_mem_ar, "ن": fin_nun_ar, "ي": "י", "ى": "א", "ء": "א",
         "و": "ו", "َ": "ַ", "ِ": "ִ", "ُ": ["ֻ", "וּ"], "ْ": "ְ"}
# Done: duplicate key, "ُ": "וּ"
# Done: Add final letter rule for סופיות

# Done: add he/ar vowels
HE_EN = {"א": ["a", "ā", "e"], "או": ["ū", "o", "u"], "אי": ["ʾI", "I"], "ב": ["b", "v"], "בּ": "b", "ג": "g",
         "ג'": ["g", "j"], "ד": ["d", "ḍ"], "ד'": "dh", "ה": heh_en, "ו": ["u", "o"], "ז": ["z", "s"], "ח": "ḥ",
         "ח'": "kh", "ט": ["t", "ṭ"], "י": "ī", "כ": ["k", "c", "q"], "ך": "kh", "ל": "l", "מ": "m", "ם": "m", "נ": "n",
         "ן": "n", "ס": "s", "ע": "ʿ", "ע'": "gh", "פ": ["f", "p"], "ף": "f", "פּ": "p", "ףּ": "p",
         "צ": ["ṣ", "ts", "z"], "ץ": ["ṣ", "ts", "z"], "ז'": ["ẓ", "z", "s"], "צ'": ["ḍ", "ch"], "ץ'": ["ḍ", "ch"],
         "ק": ["k", "q", "c"], "ר": "r", "ש": ["ṣ", "sh"], "ת": "t", "ת'": ["th", "ẓ"], "ַ": "a", "ִ": "i", "ֻ": "u",
         "-": "-", "'": "", "]": "", "׳": "", "[": "",
         "וּ": "u"}
# done: duplicate key, "א":"ʾI","א":"ī","א":"ā","ה":"ah","ט":"ṭ","ת',ד'":"ẓ","ד":"ḍ",
# Done when ה is at the end of a word, change to 'a' (one option) and another to 'e'

AR_EN = {"ال": "al", "ة": "ah", "ع": "ʿ", "طّ": "ṭṭ", "غ": "gh", "ش": "sh", "ذ": "dh", "خ": "kh", "ث": "th", "ص": "ṣ",
         "د": "d", "ا": "ā", "و": "ū", "أ": "a", "آ": "ʼā", "ب": "b", "ت": "t", "پ": "p", "چ": "g", "ه": "h", "ر": "r",
         "ز": "z", "ج": "j", "س": "s", "ض": "ḍ", "ط": "ṭ", "ظ": "ẓ", "ف": "f", "ق": "q", "ك": "k", "ل": "l", "م": "m",
         "ن": "n", "ح": "ḥ", "ي": "ī", "إ": ["ʾi", "ī"], "ى": "a", "ء": "ʾ", "َ": "a", "ِ": "i", "ُ": "u", "ْ": "ʼ"}


# Done: duplicate key ,"إ":"ī"


def non_vowel_at_start(word, position):  # Vav at the beginning of a word is pronounced like V. Otherwise like o or u
    if len(word) > 0 and position == 0:
        return False
    return True


def vowel_at_end(word, position):  # Heh at the end of a word is not pronounced
    if len(word) > 1 and position == len(word) - 1:
        return True
    return False


heb_vowels = {"א": non_vowel_at_start, "וּ": True, "ו": non_vowel_at_start, "י": non_vowel_at_start, "ה": vowel_at_end,
              "ַ": True, "ִ": True, "ֻ": True}
ar_vowels = {"ي": True, "ى": True, "ء": True, "و": non_vowel_at_start, "َ": True, "ِ": True, "ُ": True, "ْ": True}
en_vowels = {"o": non_vowel_at_start, "u": non_vowel_at_start, "O": non_vowel_at_start, "U": non_vowel_at_start
    , "a": non_vowel_at_start, "e": non_vowel_at_start, "i": non_vowel_at_start}

table_lookup = {('he', 'ar'): HE_AR, ('ar', 'he'): AR_HE, ('he', 'en'): HE_EN, ('ar', 'en'): AR_EN,
                'he': heb_vowels, 'ar': ar_vowels, 'en': en_vowels}



import unittest
from translit_me.transliterator import CharTrie


class TestCharTrie(unittest.TestCase):
    def test_single(self):
        ct = CharTrie()
        ct.add('tom')
        ct.add('er')
        res = ct.get_all()
        print(res)
        self.assertEqual(res, ['tomer'])  # add assertion here

    def test_split(self):
        ct = CharTrie()
        ct.add('tom')
        ct.split(['e', 'a'])
        ct.add('r')
        res = ct.get_all()
        print(res)
        self.assertEqual(res, ['tomer', 'tomar'])  # add assertion here


if __name__ == '__main__':
    unittest.main()



import json
from json import JSONEncoder

from fastapi import FastAPI, Request, Response

from transliterator import transliterate, remove_vowels as rv
from lang_tables import *

app = FastAPI()

lang_tables = {('he', 'ar'): HE_AR, ('ar', 'he'): AR_HE, ('he', 'en'): HE_EN, ('ar', 'en'): AR_EN}


@app.post("/")
async def home(request: Request):
    body = await request.json()
    # ToDo tests input for type list
    # ToDo tests input for language
    from_lang = body['from_lang']
    to_lang = body['to_lang']
    # ToDo tests existing table for requested lang pair
    table = lang_tables[(from_lang, to_lang)]
    res = transliterate(body['data'], table)
    print('[INFO] transliteration result: {}'.format(res))
    json_str = json.dumps({"transliterations": res}, cls=JSONEncoder).encode('utf-8')
    return Response(media_type="application/json", content=json_str)


@app.post("/remove_vowels")
async def home(request: Request):
    body = await request.json()
    # ToDo tests input for type list
    # ToDo tests input for language
    from_lang = body['lang']
    # ToDo tests existing table for requested lang pair
    table = table_lookup[from_lang]
    res = []
    for name in body['data']:
        res.append(rv(name, table))
    print('[INFO] vowel removal result: {}'.format(res))
    json_str = json.dumps({"names": res}, cls=JSONEncoder).encode('utf-8')
    return Response(media_type="application/json", content=json_str)


if __name__ == '__main__':
    import uvicorn

    uvicorn.run(app)



class CharTrie:

    def __init__(self):
        self.common = ""  # common part of the tree reduced to a string
        self.trie = []  # list of deviations (also CharTries)

    def add(self, s: str):
        """
        Add string, will be added to all variants
        :param s: string to add to current Trie
        :return: nothing
        """
        if len(self.trie) == 0:
            self.common += s
        else:
            for t in self.trie:
                t.add(s)

    def split(self, strings: list):
        """
        Split trie by multiple options
        :param strings: list of strings to be used to create the split
        :return: nothing
        """
        if len(self.trie) == 0:  # no sub-tries yet
            for s in strings:
                c = CharTrie()
                c.add(s)
                self.trie.append(c)
        else:
            t: CharTrie
            for t in self.trie:
                t.split(strings)

    def get_all(self):
        """
        Traverses the trie and returns all words that cover a path from the root to the leaves
        :return: list of words
        """
        if len(self.trie) == 0:  # Leaf node
            return [self.common]

        else:
            res = []
            t: CharTrie
            for t in self.trie:
                name: str
                for name in t.get_all():
                    res.append(self.common + name)
            return res


def transliterate(source_list, table: dict):
    """
    Transliterates names using a supplied transliteration table.
    :param source_list: list of names to transliterate
    :param table: conversion table as dictionary where each character in
                  each name of source_list has a corresponding entry
    :return: list of transliterated names in the same order
    """

    # print("[INFO] received: {}".format(source_list))
    res = []
    for name in source_list:
        word_arrays = []
        for word in name.split():
            word_arrays.append(transliterate_word(word, table))

        if len(word_arrays) == 1:
            res += word_arrays[0]
        else:
            for word in word_arrays[0]:
                res += recursive_word_assembler(word, word_arrays[1:])

    return res


def recursive_word_assembler(current_word, remaining_arrays):
    res = []
    arr = []
    if len(remaining_arrays) == 1:
        arr = remaining_arrays[0]
    else:
        for word in remaining_arrays[0]:
            arr += recursive_word_assembler(word, remaining_arrays[1:])

    for word in arr:
        res.append(current_word + " " + word)

    return res


def transliterate_word(name, table):
    new_name_trie = CharTrie()
    skip = 0
    for i in range(len(name)):  # ToDo split words on spaces
        if skip > 0:  # need to skip one or more characters due to character combinations
            skip -= 1
            continue
        if name[i: i + 3] in table.keys() and i + 2 < len(name):
            to_add = table[name[i: i + 3]]
            if type(to_add) is list:
                new_name_trie.split(to_add)
            else:
                new_name_trie.add(to_add)
            # if callable(to_add): # resolved by a function
            skip = 2
        elif name[i: i + 2] in table.keys() and i + 1 < len(name):
            to_add = table[name[i: i + 2]]
            if type(to_add) is list:
                new_name_trie.split(to_add)
            else:
                new_name_trie.add(to_add)
            skip = 1
        elif name[i] in table.keys():
            to_add = table[name[i]]
            if callable(to_add):  # resolved by a function
                to_add = table[name[i]](name, i)
            if type(to_add) is list:
                new_name_trie.split(to_add)
            else:
                new_name_trie.add(to_add)
            skip = 0
        else:
            print("[WARN] {} not found in language table for {}".format(name[i], name))
            skip = 0
    words = []
    for nn in new_name_trie.get_all():
        words.append(nn)
    return words


def remove_vowels(in_name: str, table):
    res = []
    for word in in_name.split(' '):  # split words on spaces
        new_word = ""
        skip = 0
        for i in range(len(word)):
            if skip > 0:  # need to skip one or more characters due to character combinations
                skip -= 1
                continue
            if word[i: i + 3] in table.keys() and i + 2 < len(word):
                to_remove = table[word[i: i + 3]]
                if not to_remove:
                    new_word += word[i: i + 3]
                # if callable(to_remove): # resolved by a function
                skip = 2
            elif word[i: i + 2] in table.keys() and i + 1 < len(word):
                to_remove = table[word[i: i + 2]]
                if not to_remove:
                    new_word += word[i: i + 2]
                skip = 1
            elif word[i] in table.keys():
                to_remove = table[word[i]]
                if callable(to_remove):  # resolved by a function
                    to_remove = table[word[i]](word, i)
                    if not to_remove:
                        new_word += word[i]
            else:
                new_word += word[i] # print("[WARN] {} not found in language table for {}".format(word[i], word))
                skip = 0
        res.append(new_word)
    return " ".join(res)
